# TODO: asmith 20/03/2020
# merge this file and Classes with those brought over from the mapactionpy_arcmap module

import json


class MapRecipe:
    '''
        Opens the recipe file specified by recipe_json_path and creates a MapRecipe object accordingly.
        If str_def is not None then this is treated as a string representation of the json recipe.
        `recipeJsonFile` is ignored if str_def is not None. This is primarily used for testing.
    '''

    def __init__(self, recipe_json_path, str_def=None):
        self.recipe_json_path = recipe_json_path
        self.title = ""
        self.layers = []

        if str_def:
            json_contents = json.loads(str_def)
        else:
            with open(self.recipe_json_path) as json_file:
                json_contents = json.load(json_file)

        self.title = json_contents['title']
        for layer in json_contents['layers']:
            self.layers.append(LayerSpec(layer))

    def __eq__(self, other):
        if not self.title == other.title:
            return False

        if not len(self.layers) == len(other.layers):
            return False

        listcomp = list(map(lambda sl, ol: sl == ol, self.layers, other.layers))
        return all(listcomp)

    # See https://stackoverflow.com/a/25176504/3837936
    def __ne__(self, other):
        """Overrides the default implementation (unnecessary in Python 3)"""
        return not self.__eq__(other)


class LayerSpec:
    def __init__(self, spec):
        self.map_frame = spec['map_frame']
        self.layer_group = spec['layer_group']
        self.layer_display_name = spec['layer_display_name']
        self.search_definition = spec['search_definition']
        self.data_source_path = spec['data_source_path']
        self.data_name = spec['data_name']
        self.rendering = spec['rendering']
        self.definition_query = spec['definition_query']
        self.visible = spec['visible']

    def __eq__(self, other):
        comp = [
            self.data_name == other.data_name,
            self.map_frame == other.map_frame,
            self.layer_group == other.layer_group,
            self.layer_display_name == other.layer_display_name,
            self.search_definition == other.search_definition,
            self.data_source_path == other.data_source_path,
            self.rendering == other.rendering,
            self.definition_query == other.definition_query,
            self.visible == other.visible
        ]

        return all(comp)

    def __ne__(self, other):
        """Overrides the default implementation (unnecessary in Python 3)"""
        return not self.__eq__(other)



from unittest import TestCase
import mapactionpy_controller.data_search as data_search
from mapactionpy_controller.product_bundle_definition import MapRecipe
import sys
import os
import six

# works differently for python 2.7 and python 3.x
if six.PY2:
    import mock  # noqa: F401
    from mock import mock_open, patch
else:
    from unittest import mock  # noqa: F401
    from unittest.mock import mock_open, patch  # noqa: F401


class TestDataSearch(TestCase):

    def setUp(self):
        self.parent_dir = os.path.dirname(os.path.dirname(os.path.realpath(__file__)))
        self.cmf_descriptor_path = os.path.join(self.parent_dir, 'example', 'cmf_description_flat_test.json')
        self.event_descriptor_path = os.path.join(self.parent_dir, 'example', 'event_description.json')
        self.recipe_file = os.path.join(self.parent_dir, 'example', 'product_bundle_example.json')
        self.non_existant_file = os.path.join(self.parent_dir, 'example', 'non-existant-file.json')

        # self.ds = data_search.DataSearch(cmf_descriptor_path)

    def test_args(self):
        sys.argv[1:] = ['--event', self.event_descriptor_path,
                        '--recipe-file', self.recipe_file,
                        '--output-file', 'somefile']
        args = data_search.get_args()
        self.assertEqual(self.event_descriptor_path, args.event_path)
        self.assertEqual(self.recipe_file, args.recipe_file)
        self.assertEqual('somefile', args.output_file)

    def test_non_existant_files_args(self):
        sys.argv[1:] = ['--event', self.event_descriptor_path,
                        '--recipe-file', self.non_existant_file]

        with self.assertRaises(SystemExit):
            data_search.get_args()

    # @mock.patch('json.dump')
    # @mock.patch('builtins.open', new_callable=mock_open())
    def test_data_search_main(self):
        output_file_for_testing = os.path.join(
            self.parent_dir, 'tests', 'testfiles', 'delete-me-test-output-file.json')

        sys.argv[1:] = ['--event', self.event_descriptor_path,
                        '--recipe-file', self.recipe_file,
                        '--output-file', output_file_for_testing]

        # check that the output file doesn't already exist. Run the main method and then check that it does exist
        # afterwards.
        if os.path.exists(output_file_for_testing):
            os.remove(output_file_for_testing)

        self.assertFalse(os.path.exists(output_file_for_testing))
        data_search.main()
        self.assertTrue(os.path.exists(output_file_for_testing))

        # In this case we don't expect many changes to the input and output recipes
        # - The data serach to find anything, therefore the no changes to the data_source_path
        # - The data_search regexs should be updated with the country ISO3 code
        # - The changes title remains the same
        # - The number of layers remains the same
        input_recipe = MapRecipe(self.recipe_file)
        output_recipe = MapRecipe(output_file_for_testing)

        print("input {} output {}".format(len(output_recipe.layers), len(input_recipe.layers)))

        self.assertEqual(input_recipe.title, output_recipe.title)
        self.assertEqual(len(output_recipe.layers), len(input_recipe.layers))

        # Finally remove the file so that it isn't present for future tests
        os.remove(output_file_for_testing)
        self.assertFalse(os.path.exists(output_file_for_testing))



import json
import os
# from jsonschema import validate


class CrashMoveFolder:
    def __init__(self, cmf_path, verify_on_creation=True):

        self.path = os.path.dirname(cmf_path)

        with open(cmf_path, 'r') as f:
            obj = json.loads(f.read())

            # Doubtless there is a more elegant way to do this.
            # 8 directories (alphabetical order just for readability)
            self.original_data = os.path.join(self.path, obj['original_data'])
            self.active_data = os.path.join(self.path, obj['active_data'])
            self.layer_rendering = os.path.join(self.path, obj['layer_rendering'])
            self.legend_images = os.path.join(self.path, obj['legend_images'])
            self.mxd_templates = os.path.join(self.path, obj['mxd_templates'])
            self.mxd_products = os.path.join(self.path, obj['mxd_products'])
            self.qgis_templates = os.path.join(self.path, obj['qgis_templates'])
            self.export_dir = os.path.join(self.path, obj['export_dir'])
            # 5 files (alphabetical order just for readablity)
            self.data_nc_definition = os.path.join(self.path, obj['data_nc_definition'])
            self.layer_nc_definition = os.path.join(self.path, obj['layer_nc_definition'])
            self.layer_properties = os.path.join(self.path, obj['layer_properties'])
            self.map_definitions = os.path.join(self.path, obj['map_definitions'])
            self.mxd_nc_definition = os.path.join(self.path, obj['mxd_nc_definition'])
            # others
            self.arcgis_version = obj['arcgis_version']
            # self.categories = obj['categories']

        if verify_on_creation and (not self.verify_paths()):
            failing_paths = [path for path, valid in self._get_path_verification_as_dict().items() if not valid]
            failing_paths_str = "\n\t".join(failing_paths)
            raise ValueError("Unable to verify existence of all files and directories defined in "
                             "CrashMoveFolder '{}'. The values for these parameters could not be located:\n\t"
                             "{}".format(cmf_path, failing_paths_str))

    def _get_path_verification_as_dict(self):
        results = {}

        # 8 dirs (alphabetical order just for readability)
        results['active_data'] = os.path.isdir(self.active_data)
        results['export_dir'] = os.path.isdir(self.export_dir)
        results['layer_rendering'] = os.path.isdir(self.layer_rendering)
        results['legend_images'] = os.path.isdir(self.legend_images)
        results['mxd_products'] = os.path.isdir(self.mxd_products)
        results['mxd_templates'] = os.path.isdir(self.mxd_templates)
        results['original_data'] = os.path.isdir(self.original_data)
        results['qgis_templates'] = os.path.isdir(self.qgis_templates)
        # 5 files (alphabetical order just for readability)
        results['data_nc_definition'] = os.path.exists(self.data_nc_definition)
        results['layer_nc_definition'] = os.path.exists(self.layer_nc_definition)
        results['layer_properties'] = os.path.exists(self.layer_properties)
        results['map_definitions'] = os.path.exists(self.map_definitions)
        results['mxd_nc_definition'] = os.path.exists(self.mxd_nc_definition)

        return results

    def verify_paths(self):
        # return all(all(_verify_paths().values()), self.verify_mxds())
        return all(self._get_path_verification_as_dict().values())

    def verify_mxds(self):
        result = True
        for category in (self.categories):
            for orientation in ['landscape', 'portrait']:
                templateFileName = self.arcgis_version + "_" + category + "_" + orientation

                if (category == "reference"):
                    templateFileName = templateFileName + "_bottom"
                templateFileName = templateFileName + ".mxd"
                if not os.path.exists(self.mxd_templates):
                    result = False
        return result



from unittest import TestCase
import mapactionpy_controller.check_naming_convention as check_naming_convention
import sys
import os
import six

# works differently for python 2.7 and python 3.x
if six.PY2:
    import mock  # noqa: F401
    from mock import mock_open, patch
else:
    from unittest import mock  # noqa: F401
    from unittest.mock import mock_open, patch  # noqa: F401


class TestCheckNamingConventionTool(TestCase):

    def setUp(self):
        self.parent_dir = os.path.dirname(
            os.path.dirname(os.path.realpath(__file__)))
        self.cmf_descriptor_path = os.path.join(
            self.parent_dir, 'example', 'cmf_description_flat_test.json')

    def test_get_args(self):
        sys.argv[1:] = [self.cmf_descriptor_path]

        args = check_naming_convention.get_args()
        self.assertEqual(self.cmf_descriptor_path, args.cmf_config_path)

        sys.argv[1:] = []
        with self.assertRaises(SystemExit):
            check_naming_convention.get_args()

    def test_check_naming_convention_main(self):
        sys.argv[1:] = [self.cmf_descriptor_path]

        self.assertEqual(check_naming_convention.main(), 0)



import json
import os
from mapactionpy_controller.map_layer import MapLayer
from mapactionpy_controller.crash_move_folder import CrashMoveFolder


class LayerProperties:
    """
    LayerProperties

    It is expected that a seperate LayerProperties object will be created for each layer_rendering
    file type.
    """

    def __init__(self, cmf, extension, verify_on_creation=True):
        """
        Positional Arguments:
            * cmf: Either a CrashMoveFolder object or a path to a cmf_description.json file. If it
                   is a CrashMoveFolder object and cmf.verify_paths() returns False then an
                   ValueError exception will be rasied.
            * extension: The file extension of the layer_rendering file type. (typically `.lyr` for
                         ESRI Layer files, `'.qml` for QGIS style)

        Optional Named Arguments
            verify_on_creation: True by default. If True then
            `verify_match_with_layer_rendering_dir()` will be called from the constructor.

        """
        try:
            if cmf.verify_paths():
                self.cmf = cmf
            else:
                raise ValueError('The `cmf` parameter for LayerProperties.__init__() can only accept'
                                 'values where the paths verify. eg `cmf.verify_paths() == True`.'
                                 'The value passed in this case failed this test')
        except AttributeError:
            self.cmf = CrashMoveFolder(cmf, verify_on_creation=True)

        self.extension = extension
        self.properties = {}  # Dictionary
        self._parse()

        if verify_on_creation:
            lp_only, files_only = self.get_difference_with_layer_rendering_dir()
            if len(lp_only) or len(files_only):
                msg = self._get_verify_failure_message(lp_only, files_only)
                raise ValueError(msg)

    def _get_verify_failure_message(self, lp_only, files_only):
        msg = ('There is a mismatch between:\n'
               ' (a) The layers described in the layer_properties.json file "{}"\n'
               ' and (b) The layers files, with file extension "{}", listed in the'
               ' `cmf.layer_rendering` directory "{}".\n'
               ' Either ensure that these files match, or create the LayerProperties object using the'
               ' parameter `verify_on_creation=False`\n'.format(
                   self.cmf.layer_properties,
                   self.extension,
                   self.cmf.layer_rendering
               ))

        if len(lp_only):
            msg = msg + "\nThe following layers are only in layer properties json file:\n\t"
            msg = msg + "\n\t".join(lp_only)
        if len(files_only):
            msg = msg + "\nThe following files are only in layer rendering directory:\n\t"
            msg = msg + "\n\t".join(files_only)

        return msg

    def _parse(self):
        """
        Reads layer properties file
        """
        with open(self.cmf.layer_properties) as json_file:
            jsonContents = json.load(json_file)
            for layer in jsonContents['layerProperties']:
                mapLayer = MapLayer(layer)
                self.properties[mapLayer.layerName] = mapLayer

    def get_difference_with_layer_rendering_dir(self):
        """
        Ensures that there is a one-to-one correspondance between
        * The layers described in the layer_properties.json file
        * The layers files, with the relevant file extension, listed in the `cmf.layer_rendering` directory
        By default this method is called from the constructor. It can also be called at any later time, in case the
        contents of the `cmf.layer_rendering` directory has changes on disk.
        NB: Changes to the `layer_properties.json` on disk are NOT accomadated but repeated calling of this method.

        :returns: A tuple of sets:
            * The first set is the set of layers which are in Layer Properties json file, but are in the Layer
              Rendering Directory.
            * The secound set is the set of layers which are in the Layer Rendering Directory, but not in the in Layer
              Properties json file.
        In the "ideal" senario, both sets both be empty.
        """
        lp_unique_lyrs = set()
        files_unique = set()

        for layer_name in self.properties:
            lp_unique_lyrs.add(layer_name)

        dir_content = os.listdir(self.cmf.layer_rendering)
        for f in dir_content:
            f_path = os.path.join(self.cmf.layer_rendering, f)
            filename, fileext = os.path.splitext(f)
            if (os.path.isfile(f_path)) and (fileext == self.extension):
                files_unique.add(filename)

        # sym_diff = lp_unique_lyrs.symmetric_difference(files_unique)
        lp_only = lp_unique_lyrs.difference(files_unique)
        files_only = files_unique.difference(lp_unique_lyrs)

        return lp_only, files_only



from mapactionpy_controller.map_recipe import MapRecipe
import json


class MapCookbook:
    """
    MapCookbook - Contains recipes for Map Products
    """

    def __init__(self, cookbookJsonFile):
        """
        Sets path for Map Cookbook json file.
        Creates empty list of of products.

        Arguments:
           cookbookJsonFile {str} -- path to Map Cookbook json file "mapCookbook.json"
        """
        # @TODO Add validation +
        # pass in LayerProperties object, and validate
        self.cookbookJsonFile = cookbookJsonFile
        self.products = {}
        self._parse()

    def _parse(self):
        """
        Reads product "recipes" from Map Cookbook json file
        """
        with open(self.cookbookJsonFile) as json_file:
            jsonContents = json.load(json_file)
            for recipe in jsonContents['recipes']:
                rec = MapRecipe(recipe)
                self.products[recipe['product']] = rec



from unittest import TestCase
import mapactionpy_controller.event as event
import os


class TestEvent(TestCase):

    def setUp(self):
        self.parent_dir = os.path.dirname(
            os.path.dirname(os.path.realpath(__file__)))
        self.event_descriptor_path = os.path.join(
            self.parent_dir, 'example', 'event_description.json')
        self.failing_event_descriptor_path = os.path.join(
            self.parent_dir, 'tests', 'testfiles', 'fixture_event_description_missing_operation_id.json')

    def test_constructor(self):
        # test creation when using a valid json description:
        test_cmf = event.Event(self.event_descriptor_path)
        self.assertIsInstance(test_cmf, event.Event)

        # test exception is raised when passed a non-existant file
        self.assertRaises(IOError, event.Event, '/path/to/nonexistant/file.json')
        # test exception is raised for an invalid json file:
        self.assertRaises(KeyError, event.Event, self.failing_event_descriptor_path)




class MapRecipe:
    """
    MapRecipe - Ordered list of layers for each Map Product
    """

    # TODO: asmith 2020/03/06
    # Please could we use a more meaningful name than "row" for this parameter? Isn't it a dict?
    def __init__(self, row):
        self.mapnumber = row["mapnumber"]
        self.category = row["category"]
        self.export = row["export"]
        self.product = row["product"]
        self.layers = row["layers"]
        self.summary = row["summary"]
        self.hasQueryColumnName = self.containsQueryColumn()

    def containsQueryColumn(self):
        hasQueryColumnName = False
        for layer in self.layers:
            # TODO asmith 2020/03/06
            # Is this a terser way of achieving the same thing?
            # ```
            #    if 'columnName' in layer:
            # ```
            if (layer.get('columnName', None) is not None):
                hasQueryColumnName = True
                break
        return hasQueryColumnName



from mapactionpy_controller.label_class import LabelClass


class MapLayer:
    def __init__(self, row):
        """Constructor.  Creates an instance of layer properties

        Arguments:
            row {dict} -- From the layerProperties.json file
        """
        self.mapFrame = row["MapFrame"]
        self.layerName = row["LayerName"]
        self.regExp = row["RegExp"]
        self.definitionQuery = row["DefinitionQuery"]
        self.display = row["Display"]
        self.addToLegend = row["AddToLegend"]
        self.labelClasses = list()
        for labelClass in row["LabelClasses"]:
            self.labelClasses.append(LabelClass(labelClass))



import os
import sys
from unittest import TestCase

import mapactionpy_controller.config_verify as config_verify


class TestConfigVerify(TestCase):

    def setUp(self):
        self.parent_dir = os.path.dirname(os.path.dirname(os.path.realpath(__file__)))
        self.path_to_valid_cmf_des = os.path.join(self.parent_dir, 'example', 'cmf_description_flat_test.json')
        self.path_to_invalid_cmf_des = os.path.join(
            self.parent_dir, 'tests', 'testfiles', 'fixture_cmf_description_one_file_and_one_dir_not_valid.json')

    def test_check_cmf_description(self):
        # No parameter given at commandline
        sys.argv[1:] = []
        with self.assertRaises(SystemExit):
            config_verify.run_checks(None)

        # A valid cmf description
        sys.argv[1:] = ['--cmf', self.path_to_valid_cmf_des, 'cmf-only']
        try:
            config_verify.run_checks(None)
            self.assertTrue(True)
        except SystemExit as se:
            self.fail(se)

        # An Invalid cmf description
        sys.argv[1:] = [self.path_to_invalid_cmf_des, 'cmf-only']
        with self.assertRaises(SystemExit):
            config_verify.run_checks(None)

    def test_check_lyr_props_vs_rendering_dir(self):
        pass

    def test_check_lyr_props_vs_map_cookbook(self):
        pass



import distutils.version as version
import importlib
import subprocess
from setuptools import setup, find_packages
from os import path, environ


def readme():
    here = path.abspath(path.dirname(__file__))
    with open(path.join(here, 'README.md')) as f:
        return f.read()


_base_version = '0.11'


def _get_version_number():
    travis_build = environ.get('TRAVIS_BUILD_NUMBER')
    travis_tag = environ.get('TRAVIS_TAG')

    if travis_build:
        if travis_tag:
            return travis_tag
        else:
            return '{}.dev{}'.format(_base_version, travis_build)
    else:
        try:
            ver = subprocess.check_output(['git', 'rev-parse', '--short', 'HEAD'])
            return '{}+local.{}'.format(_base_version, ver.decode('ascii').strip())
        except Exception:
            return ''


def get_install_requires():
    dependancies = [
        'jsonpickle',
        'six'
    ]
    dependancies.append(get_gis_environment())

    return dependancies


def get_gis_environment():
    gis_dependancies = []
    v = version.LooseVersion(_base_version)
    next_minor_version = '{}.{}'.format(v.version[0], v.version[1]+1)
    print('current version: {},   next minor version: {}'.format(_base_version, next_minor_version))

    # The key is the module name that is detected
    # The value is the module name that will be installed if the key is detected.
    possible_envs = {
        'arcpy': 'mapactionpy_arcmap',
        'qgis.core': 'mapactionpy_qgis'
    }

    for env_mod, target_mod in possible_envs.items():
        try:
            importlib.import_module(env_mod)
            gis_dependancies.append(
                '{}>={},<{}'.format(target_mod, _base_version, next_minor_version)
            )
            print('Found: {}, therefore installing {}'.format(env_mod, target_mod))
        except ImportError:
            print('Could not locate: {}'.format(env_mod))
            pass

    return gis_dependancies


setup(name='mapactionpy_controller',
      version=_get_version_number(),
      description='Controls the workflow of map and infographic production',
      long_description=readme(),
      long_description_content_type="text/markdown",
      url='http://github.com/mapaction/mapactionpy_controller',
      author='MapAction',
      author_email='github@mapaction.com',
      license='GPL3',
      packages=find_packages(),
      install_requires=get_install_requires(),
      test_suite='unittest',
      tests_require=['unittest'],
      zip_safe=False,
      classifiers=[
          "Development Status :: 2 - Pre-Alpha",
          "Programming Language :: Python :: 2.7",
          "Programming Language :: Python :: 3",
          "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
          "Operating System :: OS Independent",
      ])



from mapactionpy_controller.map_cookbook import MapCookbook
from mapactionpy_controller.layer_properties import LayerProperties
from mapactionpy_controller.crash_move_folder import CrashMoveFolder
import os
import argparse


def is_valid_file(parser, arg):
    if not os.path.exists(arg):
        parser.error("The file %s does not exist!" % arg)
        return False
    else:
        return arg


def is_valid_directory(parser, arg):
    if os.path.isdir(arg):
        return arg
    else:
        parser.error("The directory %s does not exist!" % arg)
        return False


class ConfigVerifier():
    def __init__(self):
        pass

    def check_cmf_description(self, args):
        try:
            CrashMoveFolder(args.cmf_desc)
            print('The Crash Move Folder description file open correctly:\n"{}"\n'.format(
                args.cmf_desc
            ))
        except ValueError as ve:
            print(ve.message)
            exit(1)

    def get_unique_lyr_names(self, cookbook, lyr_props):
        cb_unique_lyrs = set()
        lp_unique_lyrs = set()

        for recipe in cookbook.products.values():
            for l in recipe.layers:
                # print(l['name'], l)
                cb_unique_lyrs.add(l['name'])

        for l in lyr_props.properties:
            # print l
            lp_unique_lyrs.add(l)

        return (cb_unique_lyrs, lp_unique_lyrs)

    def check_lyr_props_vs_rendering_dir(self, args):
        try:
            cmf = CrashMoveFolder(args.cmf_desc)
            LayerProperties(cmf, args.layer_file_extension, verify_on_creation=True)
            print('No inconsistancy detected between:\n'
                  ' * the contents of the layer properties json file:\n\t{props}\n'
                  ' * and layer rendering dir:\n\t{render}\n'.format(
                      props=cmf.layer_properties,
                      render=cmf.layer_rendering
                  ))
        except ValueError as ve:
            print(ve.message)
            exit(1)

    def check_lyr_props_vs_map_cookbook(self, args):
        cmf = CrashMoveFolder(args.cmf_desc)
        lyrs = LayerProperties(cmf, '', verify_on_creation=False)
        cb = MapCookbook(cmf.map_definitions)
        cb_unique_lyrs, lp_unique_lyrs = self.get_unique_lyr_names(cb, lyrs)

        cb_only = cb_unique_lyrs.difference(lp_unique_lyrs)
        lp_only = lp_unique_lyrs.difference(cb_unique_lyrs)

        if len(cb_only) or len(lp_only):
            msg = ('There is a mismatch between the layer_properties.json file:\n\t"{}"\n'
                   'and the MapCookbook.json file:\n\t"{}"\n'
                   'One or more layer names occur in only one of these files.\n'.format(
                       cmf.layer_properties,
                       cmf.map_definitions
                   ))
            if len(cb_only):
                msg = msg + 'These layers are only mentioned in the MapCookbook json file and not in Layer'
                msg = msg + ' Properties json file:\n\t'
                msg = msg + '\n\t'.join(cb_only)
            if len(lp_only):
                msg = msg + '\nThese layers are only mentioned in the Layer Properties json file and not in the'
                msg = msg + ' MapCookbook json file: \n\t'
                msg = msg + '\n\t'.join(lp_only)

            print(msg)
            print('-------------------------------------\n')
            exit(2)
        else:
            print('No inconsistancy detected between:\n'
                  ' * the contents of the layer properties json file:\n\t{props}\n'
                  ' * and the contents of the MapCookbook json:\n\t{cbook}\n'.format(
                      props=cmf.layer_properties,
                      cbook=cmf.map_definitions
                  ))
            print('-------------------------------------\n')

    def get_args(self):
        parser = argparse.ArgumentParser(
            description='This tool checks the internal self-consistency various components of the Crash Move Folder,'
                        ' including several of the contained configuration files. Use sub-commands to specify which'
                        ' checks should be completed, or the special sub-comand `all` for all checks.'
        )
        parser.add_argument("-c", "--cmf", dest="cmf_desc", required=True,
                            help="path to CMF description file", metavar="FILE",
                            type=lambda x: is_valid_file(parser, x))

        subparsers = parser.add_subparsers(title='subcommands',
                                           description='valid subcommands',
                                           help='additional help')

        # `all` to call all sub commands
        parser_all = subparsers.add_parser('all')
        parser_all.description = ('Call all of the avilable subcommands to check validity')
        parser_all.set_defaults(func=self.check_all)

        parser_cmf_only = subparsers.add_parser('cmf-only')
        parser_cmf_only.description = ('Just checks the validity of the cmf_description file. This includes checking'
                                       ' that each of the file and directory paths specified are valid')
        parser_cmf_only.set_defaults(func=self.check_cmf_description)

        parser_lp_vs_rendering = subparsers.add_parser('lp-vs-rendering')
        parser_lp_vs_rendering.description = (
            'This tool checks the internal self-consistency of the cookbook file, layerProperties file and the'
            ' layerfiles within the layerDirectory'
        )
        parser_lp_vs_rendering.add_argument(
            '-e',
            '--layer-file-extension',
            dest='layer_file_extension',
            required=True,
            help='file extension layer files which will be checked against the layer_properties.json file'
        )
        parser_lp_vs_rendering.set_defaults(func=self.check_lyr_props_vs_rendering_dir)

        parser_lp_vs_cb = subparsers.add_parser('lp-vs-cb')
        parser_lp_vs_cb.description = (
            'This tool checks the internal self-consistency of the cookbook file, layerProperties file and the'
            ' layerfiles within the layerDirectory'
        )
        parser_lp_vs_cb.set_defaults(func=self.check_lyr_props_vs_map_cookbook)

        return parser.parse_args()

    # TODO look up a better way to handle the `all` option. Is there a way to extract each of teh functions from
    # the subparsers.get_defaults() method?
    def check_all(self, args):
        # for cmd_name, argparser in self.subparsers.choices.iteritems():
        #    if not cmd_name == 'all':
        #        func = argparser.get_default('func')
        #        func(args)
        self.check_cmf_description(args)
        self.check_lyr_props_vs_rendering_dir(args)
        self.check_lyr_props_vs_map_cookbook(args)


def run_checks(args=None):
    cv = ConfigVerifier()
    if not args:
        args = cv.get_args()
    args.func(args)


# TODO: asmith 2020/03/04
# This commandline interface and arg parser should be merged with other commandline interfaces
# such as the one in mapactionpy_controller.check_naming_convention.py
if __name__ == "__main__":
    run_checks()



import json
import os


class Event:
    def __init__(self, event_file):

        self.path = os.path.dirname(event_file)

        with open(event_file, 'r') as f:
            obj = json.loads(f.read())

            # Doubtless there is a more elegant way to do this.
            # 1x file path
            self.cmf_descriptor_path = os.path.join(self.path, obj['cmf_descriptor_path'])
            # 3x integers
            self.default_jpeg_res_dpi = int(obj['default_jpeg_res_dpi'])
            self.default_pdf_res_dpi = int(obj['default_pdf_res_dpi'])
            self.default_emf_res_dpi = int(obj['default_emf_res_dpi'])
            # 12x others
            self.operation_name = obj['operation_name']
            self.glide_number = obj['glide_number']
            self.affected_country_iso3 = obj['affected_country_iso3'].lower()
            self.time_zone = obj['time_zone']
            self.language_iso2 = obj['language_iso2']
            self.operation_id = (obj['operation_id']).lower()
            self.default_source_organisation = obj['default_source_organisation']
            self.default_source_organisation_url = obj['default_source_organisation_url']
            self.default_publishing_base_url = obj['default_publishing_base_url']
            self.deployment_primary_email = obj['deployment_primary_email']
            self.default_disclaimer_text = obj['default_disclaimer_text']
            self.default_donor_credits = obj['default_donor_credits']
            # self.donors = obj['donors']
